"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Pair = void 0;
var Pair = /** @class */ (function () {
    function Pair(first, second) {
        this.first = first;
        this.second = second;
    }
    return Pair;
}());
exports.Pair = Pair;
//# sourceMappingURL=Pair.js.map